/* 
 FilesystemToolsSample is a quick and dirty app that is just meant to show off
 the AASFilesystemTools framework. It has one window, with an NSOutlineView that
 lets you browse through your files, and an NSPathContol navigation bar to show
 you where the root of the outline view is and to let you choose another folder.
 
 We suggest you set up a "test folder" full of thousands of folders, subfolders,
 and files to browse and manipulate in your testing. Judicious use of
 "command A + command-D" will assist you in that regard.
 
 This is the controller object we've instantiated in the MainMenu.nib file.
 It is the delegate for the NSOutlineView.
 
 
 The SampleController instance has two variables besides the obvious IBOutlets:
 
 AASBasicInfoFile *rootFolder;
 This is the file object that represents the root file in the outline view.
 
 SampleItem *rootItem;
 This is the root item in our outline view. SampleItem objects will handle
 loading child items (files in the root folder) and other things that need
 to be done to represent our data in the outline view (we could do it all
 with a custom AASBasicInfoFile subclass but we really shouldn't be using
 file objects as controllers).
 
 
 On awakeFromNib, this object will set up the file manager and then set up the
 outline view to display your files. Some default settings are defined below,
 you might want to try changing them around and seeing what happens.
 */


// How often (in seconds) should we check the filesystem for changes?
#define INTERVAL	1.0

// Should files be loaded in a separate thread?
#define THREADED	NO

// Where should we start?
#define STARTPATH [@"~/" stringByExpandingTildeInPath]





#import "SampleController.h"




@implementation SampleController





- (void)awakeFromNib {
	
// Set up the file manager. This must be done before it is used for anything else
	
	// First, tell the file manager what class we want to use for the file objects.
	// Some day this example will use a custom subclass but for now, let's go with one built into the framework - AASBasicInfoFile
	[[AASFileManager defaultFileManager] setFileClass:[AASBasicInfoFile class]];
	
	// Tell the file manager how often to check for changes to the filesystem.
	// INTERVAL is defined at the top of this file. Try various values and then rename/move some of the files displayed to see the effects.
	[[AASFileManager defaultFileManager] setPollingInterval:INTERVAL];
	
	// Tell the file manager if we want to use threads when loading files.
	// THREADED is defined at the top of this file. Try it both ways to see how it works. You probably won't even notice much unless you're looking at folders with hundreds or thousands of files.
	[[AASFileManager defaultFileManager] setAsynchronous:THREADED];

	
	
// Set up some notifications
	
	// SampleItem objects (which we'll use to represent each file to the outline view) will post this notification whenever they get update notifications from the file manager.
	[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(filesHaveUpdated:) name:@"SampleItemUpdatedNotification" object:nil];
	
	// The root SampleItem object (which represents the root item in the outline view) will post this notification if the file is deleted or becomes otherwise inaccessible.
	[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(rootLost:) name:@"SampleItemRootLostNotification" object:nil];
	
	// Just so we can clean up later.
	[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(applicationWillTerminate:) name:NSApplicationWillTerminateNotification object:NSApp];

	
		
// Get things going
	
	// Our default path STARTPATH is defined at the top of this file.
	[self setUpWithRootPath:STARTPATH];
}


/* This method takes a path and sets up the data necessary for the outline view. We do this when the app first starts up and we do it whenever the user chooses a new root folder. */
- (void)setUpWithRootPath:(NSString *)root {
	SampleItem *prevRootItem = rootItem;
	AASBasicInfoFile *prevRootFolder = rootFolder;
	
	// Get the file object for the new root folder
	AASBasicInfoFile *newRootFolder = [AASBasicInfoFile fileForPath:root];
	
	
	// We can't have a symbolic link or alias as the root item for various technical reasons. *** This may no longer be a problem, we need to test this again
	// Not a major issue, just use the directory it connects to.
	// Ideally, we'd also check [newRootFolder isDirectory] and do appropriate things if it were actually a regular file. But we're lazy.
	if (![newRootFolder isRealFile]) {
		rootFolder = [AASBasicInfoFile fileForURL:[newRootFolder resolvedURL]];
		[newRootFolder release];
	} else {
		rootFolder = newRootFolder;
	}
	
	
	// Set up the root SampleItem. It will be responsible for loading its children (and so on recursively as necessary)
	rootItem = [[SampleItem alloc] initWithFile:rootFolder];
	[rootItem setRoot:YES];
	
	// Set the path control in our window to the correct path
	[rootPathControl setURL:[rootItem url]];
	
	// Now we have data, let's tell the outline view about it
	[testOutlineView reloadData];
	[testOutlineView setNeedsDisplay:YES];
	
	// Clean up the old stuff
	[prevRootItem release];
	[prevRootFolder release];
}


/* Clean up when quitting */
- (void)applicationWillTerminate:(NSNotification *)aNotification {
	[[NSNotificationCenter defaultCenter] removeObserver:self];
	
	[rootItem release];
	[rootFolder release];
	
	// Call this only after everything else
	[[AASFileManager defaultFileManager] shutdown];
}


/* Let the user choose a new root folder */
- (IBAction)chooseRootFolder:(id)sender {
	NSOpenPanel *openPanel = [NSOpenPanel openPanel];
	NSArray *filenames;
	
	[openPanel setCanChooseFiles:NO];
	[openPanel setCanChooseDirectories:YES];
	[openPanel setResolvesAliases:YES];
	[openPanel setAllowsMultipleSelection:NO];
	
	[openPanel runModalForTypes:nil];
	
	filenames = [openPanel filenames];
	
	if ([filenames count]) [self setUpWithRootPath:[filenames objectAtIndex:0]];
}





/* These are the NSOutlineView delegate methods that we use to populate the outline view. */

- (id)outlineView:(NSOutlineView *)outlineView child:(int)index ofItem:(id)item {
	if (!item) return rootItem;
	
	return [item childAtIndex:index];
}

- (BOOL)outlineView:(NSOutlineView *)outlineView isItemExpandable:(id)item {
	return [item isDirectory];
}

- (int)outlineView:(NSOutlineView *)outlineView numberOfChildrenOfItem:(id)item {
	if (!item) return 1;

	return [item numberOfChildren];
}

- (id)outlineView:(NSOutlineView *)outlineView objectValueForTableColumn:(NSTableColumn *)tableColumn byItem:(id)item {
	if (tableColumn == filenameColumn) return [item filename];
	
	
	if (tableColumn == typeColumn) {
		// For directories, let's return the file count rather than type
		if ([item isDirectory]) return [NSString stringWithFormat:@"%d items", [item numberOfChildren]];
		
		return [item type];
	}
	
	
	if (tableColumn == pathColumn) return [item path];
	
	// Shouldn't ever get this far because we checked for all three columns already
	return [item filename];
}






/* These next two methods get called when the appropriate notifications arrive */

- (void)filesHaveUpdated:(NSNotification *)notification {
	// Something's changed, tell the outline view to reload its data.
	[testOutlineView reloadData];
	[testOutlineView setNeedsDisplay:YES];
}


- (void)rootLost:(NSNotification *)notification {
	// Better hope this path still exists
	[self setUpWithRootPath:@"/Volumes/"];
}


@end
